(:~
 : This module creates the single edition view. According to the GET parameter
 : "edition" this module gets from edition.xqm all common text lines and the ones
 : that fit a tei:rdg are selected.
 :
 : The whole module is called by xml-utils.xqm.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 :)

xquery version "3.1";

module namespace single="http://bdn-edition.de/ns/single";

declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace html5="http://www.w3.org/1999/xhtml";

import module namespace commons="http://bdn-edition.de/ns/commons" at "commons.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace functx = "http://www.functx.com";
import module namespace id-file="https://bdn-edition.de/ns/id-file" at "id-file.xqm";

declare variable $single:id-map :=
  map:merge
  (
    for $id in doc($config:aux || "id-file.xml")//bdn:id
        return map:entry($id/string(), $id/parent::bdn:file/@uri/string())
  );
(: Wozu wird die Variable $single:id-map genutzt?! :)

declare function single:transform($nodes as node()*, $mode as xs:string*,
$file as xs:string) as node()*
{
  for $node in $nodes
  return
    (
      if ($node/@xml:id) then
        commons:make-text-anchor(string($node/@xml:id)
        (: Hier werden alle Elemente mit xml:id mit html-Ankerpunkten versehen :)
    )
    else
      (),
    typeswitch($node)
    case text() return
      let $text := text{replace($node, "@", " ")}
      return
(:          let $app := $node/following-sibling::*[1][self::tei:app]:)
(:          let $next-text := $app/following-sibling::node()[1][self::text()]:)
(:          let $next-text-first-char := substring($next-text, 1, 1):)
(:          return:)
(:              (: example: Griesbach 250bg, #a "einen neuen Zuwachs, wenn man":):)
(:              if ($app/tei:rdg[matches(@wit, $mode) and @type = "om"]:)
(:              and matches($next-text-first-char, "[,\.\?!\)]")) then:)
(:                  text{functx:right-trim($text)}:)
(:              else:)
                  $text

    case comment() return
      ()

    case element(tei:teiHeader) return
      <div>{data($mode)}</div>
      (: Wie werden die Daten aus dem tei:Header dargestellt?
      
      :)

    case element(tei:div) return
      switch($node/@type)
      case ""
        return
        element xhtml:div
        {
          single:transform($node/node(), $mode, $file)
        }
        
    (: Auskommentiert, weil es diesen div-Type nicht gibt in der KMG
      case "synopsis" return
        (
          single:transform($node/tei:head, $mode, $file),
          element xhtml:table
          {
            element xhtml:tr
            {
              element xhtml:td
              {
                single:transform($node/descendant::tei:div[@type = "letter"][1]/tei:head, $mode, $file)
              },
              element xhtml:td
              {
                single:transform($node/descendant::tei:div[@type = "letter"][2]/tei:head, $mode, $file)
              }
            },
            (: ATTENTION: this implies that the two div[@type = 'letter']
            always have the same number of ab :)
            (
              for $ab in $node/descendant::tei:div[@type = "letter"][1]/tei:ab
              return
                element xhtml:tr
                {
                  element xhtml:td
                  {
                    attribute class {"syn-td"},
                    single:transform($node/node(), $mode, $file)
                  },
                  element xhtml:td
                  {
                    attribute class {"syn-td"},
                    single:transform($node/descendant::div[@type = 'letter'][2]/ab[@xml:id = $ab/@corresp], $mode, $file)
                  }
                }
            )
          }
        )
    :)
    (: Der folgende case sollte eigentlich auskommentiert werden, da es diesen div-Type in der KMG nicht gibt. Aber dann kommt eine Fehlermeldung in "default return" am Ende der Funktion single:transform :)
      case "letter" return
        if ($node/ancestor::tei:div[@type = "synopsis"]) then
          ()
        else
          single:transform($node/node(), $mode, $file)
      default return
        element xhtml:div
        {
          attribute class {$node/@type},
          single:transform($node/node(), $mode, $file)
        }

    (: Differenzierung tei:head in h2, h3, h4, h5; tei:head in tei:table :)
    case element(tei:head) return
        if ($node/parent::tei:figure) then
            ()
        else if ($node/parent::tei:table) then
            element html5:caption
            {
                if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                single:transform($node/node(), $mode, $file)
            }
        else if ($node[@type = "sub"]) then
            (if ($node/parent::tei:div[@type = "part"]) then
                element xhtml:h2
                {
                  attribute class {"list-sub-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else if ($node/parent::tei:div[@type = "chapter"]) then
                element xhtml:h3
                {
                  attribute class {"list-sub-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else if ($node/parent::tei:div[@type = "section"]) then
                element xhtml:h4
                {
                  attribute class {"list-sub-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else
                element xhtml:h5
                {
                  attribute class {"list-sub-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            )
        else
            (if ($node/parent::tei:div[@type = "part"]) then
                element xhtml:h2
                {
                  attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else if ($node/parent::tei:div[@type = "chapter"]) then
                element xhtml:h3
                {
                  attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else if ($node/parent::tei:div[@type = "section"]) then
                element xhtml:h4
                {
                  attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else
                element xhtml:h5
                {
                  attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            )

    (: Auskommentiert, weil es diesen head-Type nicht gibt in der KMG
      switch($node/@type)
      case "main" return
        element xhtml:div
        {
          attribute class {"list-main-head"},
          single:transform($node/node(), $mode, $file)
        }
    :)
    (: Diesen head-Type gibt es in der KMG, aber wie soll er angezeigt werden?
      case "sub" return
        element xhtml:div
        {
          attribute class {"list-sub-head"},
          single:transform($node/node(), $mode, $file)
        }
      default return
    :)
    (: Auskommentiert, weil ersetzt durch Differenzierung
        element xhtml:h2
        {
          if ($node/@copyOf) then
            single:apply-corresp-node($node/@copy-of, $mode, $file, false())
          else
            single:transform($node/node(), $mode, $file)
        }
    :)

    (: Hier könnten noch die verschiedenen Anwendungsfälle spezifiziert werden :)
    case element(tei:seg) return
      if ($node/@copyOf) then
        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
      else if ($node/@type = "margin") then
            element xhtml:span
            {
                attribute class {"margin"},
                text{"{"},
                single:transform($node/node(), $mode, $file),
                text{"}"}
            }
        else if ($node/@type = "list") then
            (if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'label') then
              element xhtml:li
              {
                if ($node/@xml:id) then
                  attribute id {$node/@xml:id}
                else
                  (),
                attribute class {"labeled_item"},
                <xhtml:span class="list-label">{data($node/preceding-sibling::tei:label[@type="list"][1])}</xhtml:span>,
                if ($node/@copyOf) then
                  single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                  single:transform($node/node(), $mode, $file)
              }
            else
               element xhtml:li
              {
                if ($node/@xml:id) then
                  attribute id {$node/@xml:id}
                else
                  (),
                attribute class {"unlabeled_item"},
                if ($node/@copyOf) then
                  single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                  single:transform($node/node(), $mode, $file)
              } 
            )
      else
        single:transform($node/node(), $mode, $file)


    (: vorerst unverändert :)
    case element(tei:front) return
      element xhtml:div
      {
        attribute class {"preface"},
        single:transform($node/node(), $mode, $file)
      }

    (: Auskommentiert, da tei:titlePart in der KMG nicht existiert
    case element(tei:titlePart) return
      switch($node/@type)
      case "main" return
        element xhtml:h1
        {
          attribute class {"main-title"},
          single:transform($node/node(), $mode, $file)
        }
      case "sub" return
        element xhtml:h2
        {
          attribute class {"sub-title"},
          single:transform($node/node(), $mode, $file)
        }
      case "volume" return
        element xhtml:div
        {
          attribute class {"volume"},
          single:transform($node/node(), $mode, $file)
        }
      default return ()
      :)

    case element(tei:app) return
      (if ($node/child::*[matches(@wit, $mode)]) then
        single:transform($node/child::*[matches(@wit, $mode)], $mode, $file)
      else
        single:transform($node/child::tei:lem, $mode, $file),
      if (single:is-next-text-in-app($node)
      and single:is-next-app-omitted($node, $mode)) then
        ()
      else
        commons:make-whitespace-after($node))

    case element(tei:rdg) return
     single:transform($node/node(), $mode, $file)

    (: Auskommentiert, da dieses Element in der KMG nicht existiert
    case element(tei:rdgMarker) return
      ()
    :)

    (: Angepasst: "el" statt "gr" und hi-Bedingungen entfernt :)
    case element(tei:foreign) return
      if ($node[@xml:lang = "el"]) then
        element xhtml:span
        {
          attribute class {"hi-greek"},
          single:transform($node/node(), $mode, $file)
        }
      else
        element xhtml:span
        {
          attribute class {"foreign"},
          single:transform($node/node(), $mode, $file)
        }

    (: Auskommentiert, da dieses Element in der KMG nicht existiert
    case element(tei:aligned) return
      element xhtml:div
      {
        attribute class {$node/@rend},
        single:transform($node/node(), $mode, $file)
      }
    :)

    (: "spaced-out" in "spaced" geändert; hier müssen noch die anderen KMG-Möglichkeiten für @rend eingearbeitet werden; Frage: was macht die class "hi"?:)
    case element(tei:hi) return
      if ($node/@rend) then
        element xhtml:span
        {
          if ($node/@rend = "spaced"
          and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
              attribute class {"spaced-out spaced-out-no-right-padding"}
          else
              attribute class {$node/@rend},
          single:transform($node/node(), $mode, $file)
        }
      else
        element xhtml:span
        {
          attribute class {"hi"},
          single:transform($node/node(), $mode, $file)
        }

    (: Auskommentiert, da dieses Element in der KMG nicht existiert
    case element(tei:index) return
      ()
    :)

    (: vorerst: Vereinfachung :)
    case element(tei:label) return
      if ($node[@type = "note_anchor"]) then
          (if ($node/parent::*[@anchored = "false"]) then
              element xhtml:span
                {
                  attribute class {"anchored_false"},
                  single:transform($node/node(), $mode, $file),
                  text {" "}
                }
            else
                element xhtml:span
                {
                  attribute class {"note_anchor"},
                  single:transform($node/node(), $mode, $file),
                  text {" "}
                }
          )
      else if ($node[@type = "list"]) then
          ()
      else
        element xhtml:span
        {
          attribute class {"label-wrapper"},
          single:transform($node/node(), $mode, $file)
        }

    (: vorerst unverändert, muss noch angepasst werden :)
    case element(tei:list) return
        if ($node/@type = ("unspecified", "gloss", "index")) then
            element xhtml:ul
              {
                (if ($node/parent::tei:item) then
                  attribute class {"nested-list unlabeled_list"}
                else if ($node/ancestor::tei:rdg) then
                  attribute class {"rdg-list unlabeled_list"}
                else
                  attribute class {"text-list unlabeled_list"}),
                single:transform($node/node(), $mode, $file)
              }
        else if ($node/@type = "abbildungstext") then
            ()
        else
          element xhtml:ul
          {
            (if ($node/parent::tei:item) then
              attribute class {"nested-list labeled_list"}
            else if ($node/ancestor::tei:rdg) then
              attribute class {"rdg-list labeled_list"}
            else
              attribute class {"text-list labeled_list"}),
            single:transform($node/node(), $mode, $file)
          }


    (: vorerst unverändert :)
    case element(tei:l) return
        if ($node/parent::tei:lg/parent::tei:quote or $node/parent::tei:lg/parent::tei:q) then
            (if (not($node/preceding-sibling::tei:l)) then
                (if ($node/parent::tei:lg/parent::*[contains(@rend, "double")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-qm-low-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-qm-high-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-q-out-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-q-in-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        (if ($node/ancestor::*[@wit]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                        else
                            (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-qm-low-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-qm-high-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-g-out-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-g-in-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else
                            element xhtml:span
                            {
                                attribute class {"line quote double-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                            )
                        )
                    )
                else if ($node/parent::tei:lg/parent::*[contains(@rend, "single")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-qm-low-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-qm-high-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-q-out-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-q-in-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        (if ($node/ancestor::*[@wit]) then
                        
                        element xhtml:span
                        {
                            attribute class {"line quote single-l"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                        else
                            (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-qm-low-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-qm-high-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-g-out-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-g-in-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else
                            element xhtml:span
                            {
                                attribute class {"line quote single-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                            )
                        )
                    )
                else
                    element xhtml:span
                    {
                    attribute class {"line quote"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                    }
                )
            else if (not($node/following-sibling::tei:l)) then
                (if ($node/parent::tei:lg/parent::*[contains(@rend, "double")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-qm-low-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-qm-high-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-q-out-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-q-in-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        (if ($node/ancestor::*[@wit]) then
                        element xhtml:span
                        {
                            attribute class {"line quote double-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                        else
                            (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-qm-low-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-qm-high-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-g-out-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote double-g-in-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else
                            element xhtml:span
                            {
                                attribute class {"line quote double-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                            )
                        )
                    )
                else if ($node/parent::tei:lg/parent::*[contains(@rend, "single")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-qm-low-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-qm-high-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-q-out-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        element xhtml:span
                        {
                            attribute class {"line quote single-q-in-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        (if ($node/ancestor::*[@wit]) then
                        
                        element xhtml:span
                        {
                            attribute class {"line quote single-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                        else
                            (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-qm-low-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-qm-high-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-g-out-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                            element xhtml:span
                            {
                                attribute class {"line quote single-g-in-r"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                        else
                            element xhtml:span
                            {
                                attribute class {"line quote single-l"},
                                if ($node/@copyOf) then
                                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                                else
                                    single:transform($node/node(), $mode, $file)
                            }
                            )
                        )
                    )
                else
                    element xhtml:span
                      {
                        attribute class {"line quote"},
                        if ($node/@copyOf) then
                            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                        else
                            single:transform($node/node(), $mode, $file)
                      }
                )
            else
                element xhtml:span
                  {
                    attribute class {"line quote"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                  }
            )
        else
          element xhtml:span
          {
            attribute class {"line"},
            single:transform($node/node(), $mode, $file)
          }

    (: angepasst :)
    case element(tei:lg) return
        element xhtml:div
          {
            attribute class {"linegroup"},
            single:transform($node/node(), $mode, $file)
          }

    case element(tei:note) return
        if ($node/@type = "KMG") then
            (if ($node/ancestor::tei:p) then
              <br />
            else
              (),
            element xhtml:div
            {
                attribute class {"note kmg-note"},
                single:transform($node/node(), $mode, $file)
            })
        else if ($node/@type = "editorial") then
            (if ($node/ancestor::tei:p) then
              <br />
            else
              (),
            element xhtml:div
            {
                attribute class {"note kmg-note"},
                single:transform($node/node(), $mode, $file)
            })
        else
            (if ($node/ancestor::tei:p) then
              <br />
            else
              (),
            element xhtml:div
            {
                attribute class {"note"},
                single:transform($node/node(), $mode, $file)
            })

    (: vorerst unverändert :)
    case element(tei:ptr) return
      ()

    (: Auskommentiert, da tei:titlePage in der KMG nicht existiert
    case element(tei:titlePage) return
      element xhtml:span
      {
        attribute class {"title-page"},
        single:transform($node/node(), $mode, $file)
      }
    :)

    (: Auskommentiert, da tei:ab in der KMG nicht existiert
    case element(tei:ab) return
      if ($node[@type = "half-title"]) then
        element xhtml:div
        {
          attribute class {$node/@type},
          single:transform($node/node(), $mode, $file)
        }
      else
        element xhtml:span
        {
          attribute class {"ab"},
          single:transform($node/node(), $mode, $file)
        }
    :)

    (: vorerst unverändert :)
    case element(tei:cb) return
      element xhtml:span
      {
        attribute class {"cb"},
        string("|" || replace($node/@edRef, '[# ]+', '') || $node/@n || "|")
      }

    (: vorerst unverändert :)
    case element(tei:pb) return
      if (matches($node/@edRef, $mode)) then
        (
          commons:make-whitespace-before($node),
          element xhtml:span
          {
            attribute class {"pb"},
            concat
            (
              string("|" || replace($node/@edRef, '[# ]+', '')),
              (if ($node/@type = 'sp') then string("[") else ""),
              string($node/@n),
              (if ($node/@type = 'sp') then string("]") else ""),
              string("|")
            )
          }
        )
      else if (not($node/following-sibling::*[1][self::tei:pb])) then
        commons:make-whitespace-after($node)
      else
        ()
        
    case element(tei:milestone) return
        if ($node/@unit = "page"
        and matches($node/@edRef, $mode)) then
            (
              commons:make-whitespace-before($node),
              element xhtml:span
              {
                attribute class {"pb"},
                concat
                (
                  string("|" || replace($node/@edRef, '[# ]+', '')),
                  (if ($node/@type = 'sp') then string("[") else ""),
                  string($node/@n),
                  (if ($node/@type = 'sp') then string("]") else ""),
                  string("|")
                )
              }
            )
        else
            ()
        
              
    (:
    : Frühere Funktion für tei:milestone
    : let $edition := replace(replace($node/@edRef, "#", ""), " ", "")
    :    return
    :      if ($node/@unit = "fn-break"
    :      and $edition = $mode) then
    :        element xhtml:span
    :        {
    :          attribute class {"fn-break"},
    :          string("|" || $edition || $node/@n || "|")
    :        }
    :      else
    :          ()
    :)


    (: vorerst unverändert :)
    case element(tei:back) return
      single:transform($node/node(), $mode, $file)

    (: angepasst an kmg :)
    case element(tei:item) return
      if ($node/parent::tei:figure) then
            ()
      else if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'label') then
          element xhtml:li
          {
            if ($node/@xml:id) then
              attribute id {$node/@xml:id}
            else
              (),
            attribute class {"labeled_item"},
            <xhtml:span class="list-label">{data($node/preceding-sibling::tei:label[@type="list"][1])}</xhtml:span>,
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
          }
      else if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'app') then
          element xhtml:li
          {
            if ($node/@xml:id) then
              attribute id {$node/@xml:id}
            else
              (),
            attribute class {"labeled_item"},
            <xhtml:span class="list-label">{data($node/preceding-sibling::tei:app[1]/*[@wit="#" || $mode]/tei:label[@type="list"])}</xhtml:span>,
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
          }
      else
          element xhtml:li
          {
            if ($node/@xml:id) then
              attribute id {$node/@xml:id}
            else
              (),
            attribute class {"unlabeled_item"},
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
          }
          


    (: Unnötige Spezifizierungen auskommentiert, Attributswerte angepasst :)
    case element(tei:choice) return
        if ($node/tei:corr[@type = "editorial"]) then
            element html5:span
            {
                attribute class {"resp-tt"},
                if ($node/@copyOf) then
                  single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                  single:transform($node/node(), $mode, $file),
                element html5:span
                {
                    attribute class {"resp-tttext"},
                    "Verantwortlich: ",
                    element html5:a
                    {
                        attribute href {$node/tei:corr/@resp},
                        replace(data($node/tei:corr/@resp), "#", "")
                    }
                }
            }
        else
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
        (: if ($node//tei:abbr) then
            single:transform($node//tei:abbr, $mode, $file)
        else if ($node//tei:orig) then
        single:transform($node//tei:orig, $mode, $file)
        (: compositor's errors :)
          else if ($node/tei:corr
          and ($node/..[@type = "typo-correction"][contains(@wit, $mode)]
            or not($node/parent::tei:lem/following-sibling::tei:rdg[contains(@wit, $mode)]))) then
            (single:transform($node/tei:corr, $mode, $file),
            single:make-typo-corr($node, $mode, $file),
            (: compositor's error with authorial correction :)
            if ($node/tei:corr[@type = "authorial"]) then
              element xhtml:a
              {
                attribute href {commons:get-link-url($node, $mode)},
                <i class="fas fa-pen-fancy"></i>
              }
            else
              ()
            )
          (: authorial corrigenda :)
          else if ($node/tei:corr[@type = "errata"]) then
            (single:transform($node//tei:corr, $mode, $file),
            element xhtml:a
            {
              attribute href {commons:get-link-url($node, $mode)},
              string("@TEST@"),
              <i class="fas fa-pen-fancy"></i>
            })
          else
            () (:TODO: Satzfehler :)
            :)
    
    case element(tei:sic) return
        element html5:span
        {
            attribute class {"kmg linethrough"},
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }
    
    case element(tei:corr) return
        element html5:span
        {
            attribute class {"kmg"},
            text{" "},
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }

    (: Es kann noch andere @result außer p und item geben, insofern muss dies angepasst werden :)
    case element(tei:join) return
      switch($node/@result)
      case "p" return
        element xhtml:div
        {
          attribute class {"p"},
          single:apply-join($node, $mode, $file)
        }
      case "item" return
        element xhtml:li
        {
          attribute class {"list-item"},
          single:apply-join($node, $mode, $file)
        }
      (: the default case is that join has no @result :)
      default return
        single:apply-join($node, $mode, $file)


    (: Frage: bei xhtml:div bleiben oder zu xhtml:p ändern? Dann noch die drei @rend von tei:p einpflegen :)
    case element(tei:p) return
      (: since we have xhtml:div nested in p, e.g. for compositor's
      errors, we need to choose xhtml:div here instead of xhtml:p.
      otherwise the HTML content model is violated. :)
      if ($node/ancestor::tei:figure) then
            element html5:figcaption
            {
               single:transform($node/node(), $mode, $file) 
            }
      else
          element xhtml:div
          {
                if ($node/@rend= "blockquote") then
                    attribute class {"blockquote p"}
                else if ($node/@rend= "blockquoteitalics") then
                    attribute class {"blockquote italics p"}
                else
                    attribute class {"p"},
            (:if (preceding-sibling::p
            or (ancestor::app[@type = 'structural-variance']/parent::div[@type = 'section']
            and ancestor::div[@type = 'section']/child::*[2] != ancestor::app[@type = 'structural-variance']):)
            (: in case a paragraph starts directly with an app[@type = 'structural-variance'] :)
            (:or ancestor::app[@type = 'structural-variance']/parent::div[@type = 'section']
              and not(//*[@xml:id = $node/@copyID]/ancestor::app[@type = 'structural-variance'])
            or ancestor::app/preceding-sibling::*[1][self::p]
            or preceding-sibling::*[1][self::list]
            or preceding-sibling::*[1]/child::*[last()][self::list]
            or preceding-sibling::*[1][self::app]/lem/child::*[last()][self::list])
              then
              attribute{"style"}{"text-indent: 15px;"}
              else
              attribute{"style"}{"margin-left: 0;"},:)
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
              else
              single:transform($node/node(), $mode, $file)
            }


    (: vorerst unverändert :)
    case element(tei:ref) return
      (if (starts-with($node/@target, "#")) then
          element html5:a
            {
                attribute href {$node/[@target]},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        else
            element xhtml:span
            {
              attribute class {"ref"},
              single:transform($node/node(), $mode, $file),
              string(" "),
              element xhtml:a
              {
                attribute class {"text-link"},
                attribute href {commons:get-link-url($node, $mode)},
                <i class="fas fa-external-link-alt"></i>
              }
            },
            if (commons:starts-next-text-with-punctuation($node, $mode)) then
              ()
            else
              text{" "}
      )

    (: vorerst unverändert :)
    case element(tei:lb) return
      if ($node/preceding-sibling::*[self::tei:titlePart or self::tei:byline
      or self::tei:docImprint or self::tei:head]) then
        ()
      else
        <br />

    (: Auskommentiert, da tei:docImprint in der KMG nicht existiert
    case element(tei:docImprint) return
        (single:transform($node/node(), $mode, $file),
        <br />)
    :)

    (: Auskommentiert, da tei:titlePage in der KMG nicht existiert
    case element(tei:supplied) return
      (
        text {"["},
        single:transform($node/node(), $mode, $file),
        text{"]"}
      )
    :)

    (: vorerst unverändert :)
    case element (tei:floatingText) return
        element xhtml:div
        {
            attribute class {"floating-text"},
            single:transform($node/node(), $mode, $file)
        }


    (: vorerst unverändert :)
    case element (tei:signed) return
        element xhtml:div
        {
            attribute class {"signed"},
            single:transform($node/node(), $mode, $file)
        }

    case element(tei:quote) return
        if ($node/descendant::tei:lg) then
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        else if ($node/@rend = "none") then
            element xhtml:q
            {
                attribute class {"quote"},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        else if ($node[contains(@rend, "double")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                element xhtml:q
                {
                    attribute class {"quote double-qm-low-l double-qm-low-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "qm_high")]) then
                element xhtml:q
                {
                    attribute class {"quote double-qm-high-l double-qm-high-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }    
            else if ($node[contains(@rend, "g_out")]) then
                element xhtml:q
                {
                    attribute class {"quote double-q-out-l double-q-out-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "g_in")]) then
                element xhtml:q
                {
                    attribute class {"quote double-q-in-l double-q-in-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else
                if ($node/ancestor::*[@wit]) then
                    
                    element xhtml:q
                    {
                        attribute class {"quote double-l double-r"},
                        if ($node/@copyOf) then
                            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                        else
                            single:transform($node/node(), $mode, $file)
                    }
                else
                    (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                        element xhtml:q
                        {
                            attribute class {"quote double-qm-low-l double-qm-low-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                        element xhtml:q
                        {
                            attribute class {"quote double-qm-high-l double-qm-high-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                        element xhtml:q
                        {
                            attribute class {"quote double-g-out-l double-g-out-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                        element xhtml:q
                        {
                            attribute class {"quote double-g-in-l double-g-in-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        element xhtml:q
                        {
                            attribute class {"quote double-l double-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    )
                    
                )
        else if ($node[contains(@rend, "single")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                element xhtml:q
                {
                    attribute class {"quote single-qm-low-l single-qm-low-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "qm_high")]) then
                element xhtml:q
                {
                    attribute class {"quote single-qm-high-l single-qm-high-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }    
            else if ($node[contains(@rend, "g_out")]) then
                element xhtml:q
                {
                    attribute class {"quote single-q-out-l single-q-out-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "g_in")]) then
                element xhtml:q
                {
                    attribute class {"quote single-q-in-l single-q-in-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else
                if ($node/ancestor::*[@wit]) then
                    element xhtml:q
                        {
                            attribute class {"quote single-l single-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    
                else
                    (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                        element xhtml:q
                        {
                            attribute class {"quote single-qm-low-l single-qm-low-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                        element xhtml:q
                        {
                            attribute class {"quote single-qm-high-l single-qm-high-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                        element xhtml:q
                        {
                            attribute class {"quote single-g-out-l single-g-out-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                        element xhtml:q
                        {
                            attribute class {"quote single-g-in-l single-g-in-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        element xhtml:q
                        {
                            attribute class {"quote single-l single-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    )
                    
                )
        else
            ()
    case element(tei:q) return
        if ($node/descendant::tei:lg) then
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        else if ($node[contains(@rend, "double")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                element xhtml:q
                {
                    attribute class {"double-qm-low-l double-qm-low-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "qm_high")]) then
                element xhtml:q
                {
                    attribute class {"double-qm-high-l double-qm-high-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }    
            else if ($node[contains(@rend, "g_out")]) then
                element xhtml:q
                {
                    attribute class {"double-q-out-l double-q-out-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "g_in")]) then
                element xhtml:q
                {
                    attribute class {"double-q-in-l double-q-in-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else
                if ($node/ancestor::*[@wit]) then
                    
                    element xhtml:q
                    {
                        attribute class {"double-l double-r"},
                        if ($node/@copyOf) then
                            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                        else
                            single:transform($node/node(), $mode, $file)
                    }
                else
                    (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                        element xhtml:q
                        {
                            attribute class {"double-qm-low-l double-qm-low-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                        element xhtml:q
                        {
                            attribute class {"double-qm-high-l double-qm-high-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                        element xhtml:q
                        {
                            attribute class {"double-g-out-l double-g-out-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                        element xhtml:q
                        {
                            attribute class {"double-g-in-l double-g-in-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        element xhtml:q
                        {
                            attribute class {"double-l double-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    )
                    
                )
        else if ($node[contains(@rend, "single")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                element xhtml:q
                {
                    attribute class {"single-qm-low-l single-qm-low-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "qm_high")]) then
                element xhtml:q
                {
                    attribute class {"single-qm-high-l single-qm-high-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }    
            else if ($node[contains(@rend, "g_out")]) then
                element xhtml:q
                {
                    attribute class {"single-q-out-l single-q-out-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else if ($node[contains(@rend, "g_in")]) then
                element xhtml:q
                {
                    attribute class {"single-q-in-l single-q-in-r"},
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file)
                }
            else
                if ($node/ancestor::*[@wit]) then
                    element xhtml:q
                        {
                            attribute class {"single-l single-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    
                else
                    (if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_low")]) then
                        element xhtml:q
                        {
                            attribute class {"single-qm-low-l single-qm-low-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "qm_high")]) then
                        element xhtml:q
                        {
                            attribute class {"single-qm-high-l single-qm-high-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_out")]) then
                        element xhtml:q
                        {
                            attribute class {"single-g-out-l single-g-out-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else if (//tei:sourceDesc/tei:listWit/tei:witness[1][contains(@rend, "g_in")]) then
                        element xhtml:q
                        {
                            attribute class {"single-g-in-l single-g-in-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    else
                        element xhtml:q
                        {
                            attribute class {"single-l single-r"},
                            if ($node/@copyOf) then
                                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                            else
                                single:transform($node/node(), $mode, $file)
                        }
                    )
                    
                )
        else
            ()
    
    case element(tei:figure) return
        element html5:figure
        {
            attribute class {"image"},
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }
    
    case element(tei:graphic) return
        element html5:a
        {
            attribute href {replace($node/[@url], "textgrid:", "/exist/apps/sade/textgrid/data/kmg-images/") || ".jpg"},
            element html5:img
            {
                attribute src {replace($node/[@url], "textgrid:", "/exist/apps/sade/textgrid/data/kmg-images/") || ".jpg"},
                attribute alt {data(replace(string($node/ancestor::tei:figure/descendant::tei:figDesc), "@", " "))},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        }
    
    case element(tei:figDesc) return
        ()

    case element(tei:name) return
        if ($node[contains(@ref, "textgrid:")]) then
            element html5:a
            {
                attribute href {replace($node/[@ref], "textgrid:", "/exist/apps/sade/textgrid/data/kmg-lido/") || ".xml"},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        else if ($node[contains(@ref, "gnd:")]) then
            element html5:a
            {
                attribute href {replace($node/[@ref], "gnd:", "http://d-nb.info/gnd/")},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        else
            element html5:a
            {
                attribute href {$node/[@ref]},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
    
    case element(tei:bibl) return
        if ($node/@type = "KMG") then
            element html5:a
            {
                attribute class {"kmg"},
                attribute href {replace($node/[@corresp], "zotero:", "http://zotero.org/groups/2206325/items/")},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        else
            element html5:a
            {
                attribute href {replace($node/[@corresp], "zotero:", "http://zotero.org/groups/2206325/items/")},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }

    case element(tei:table) return
        if ($node/tei:trailer) then
            element html5:figure
            {
               attribute class {"tablefig"},
               element html5:table
               {
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file) 
               },
               element html5:figcaption
               {
                   if ($node/tei:trailer/@copyOf) then
                        single:apply-corresp-node($node/tei:trailer, $node/tei:trailer/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/tei:trailer/node(), $mode, $file) 
               }
            }
        else
            element html5:figure
            {
               attribute class {"tablefig"},
               element html5:table
               {
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file) 
               }
            }

    case element(tei:row) return
        element html5:tr
        {
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }

    case element(tei:cell) return
        if ($node/parent::tei:row/parent::tei:table/@rend = "no_border") then
            element html5:td
            {
               attribute class {"no_border"},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else if ($node/parent::tei:row/@rend = "no_border") then
            element html5:td
            {
               attribute class {"no_border"},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else if ($node/@rend) then
            element html5:td
            {
               attribute class {data($node/@rend)},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else
            element html5:td
            {
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
    
    case element(tei:trailer) return
        ()

    (: vorerst unverändert :)
    default return
      single:transform($node/node(), $mode, $file),
      commons:make-whitespace-after($node)
  )
};


(:~
 : Finds and applies the node that is referenced with @copyOf.
 : In case of tei:join we have to apply only the child nodes of the corresponding
 : nodes because tei:join defines a new resulting element in @result that makes
 : the original nodes obsolete.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @param $mode The current edition
 : @param $file
 : @param $join A flag indicating if the calling element is a tei:join
 : @return The transformed node the @copyOf refers to
 :)
declare function single:apply-corresp-node($node as node()*, $copyOf as xs:string, $mode as
xs:string, $file as xs:string, $apply-children as xs:boolean)
as node()*
{
  let $copyOf := replace($copyOf, "#", "")
  let $corresp-node := single:find-corresp-node($node, $copyOf)
  return
      if ($apply-children) then
          for $child in $corresp-node return
              single:transform($child, $mode, $file)
      else
            single:transform($corresp-node, $mode, $file)
};


(:~
 : Retrieves the target node of a given @copyOf-reference. 
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @return The node the @copyOf refers to
 :)
declare function single:find-corresp-node($node as node()*, $copyOf as xs:string) as node()?
{
  (:let $author := commons:get-author($file)
  let $copyOf := replace($copyOf, "#", "")
  let $target-file := map:get($single:id-map, $copyOf)
  let $doc := doc($config:edition || $author || "/xml/" || $target-file || ".xml")
  :)
(:   let $doc := doc($config:edition || "kmg" || "/xml/" || $file || ".xml"):)
   (: let $doc := doc(/db/apps/sade/textgrid/data/kmg/Mollenhauer-1969.xml) würde funktionieren:)
(:  return:)
(:    $doc//*[@xml:id = $copyOf]:)
    $node/root()//*[@xml:id = $copyOf]
};


(:~
 : Creates the icon for a compositor's error and initiates the construction of
 : the respective modal div.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The the current tei:choice
 : @param $mode The current edition
 : @return The modal div, a whitespace and the icon
 :)
declare function single:make-typo-corr($node as element(tei:choice),
$mode as xs:string, $file as xs:string) as node()*
{
  let $random-id := single:make-random-id()
  let $id := util:hash($random-id, "md5")
  let $edition :=
    if ($node/parent::tei:lem[not(@wit)]) then
      (: single file names don't have a hyphen, only the ones that are the
      result of semantic splitting :)
      if (matches($file, "-")) then
        let $author := substring-before($file, "-")
        let $full-file := doc($config:full-xmls || $author || "-full.xml")
        return $full-file//tei:witness[@n = "base-text"]/@xml:id/string()
      else
        $node/ancestor::tei:TEI//tei:witness[@n = "base-text"]/@xml:id
    else
      $mode
  return
    (single:make-typo-corr-modal($node, $edition, $mode, $id, $file),
    text{" "},
    <a class="comp-error" data-target="#{$id}">
      <i class="fa fa-wrench" aria-hidden="true" data-toggle="modal" />
    </a>)
};


(:~
 : Creates the modal div for a compositor's error.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The the current tei:choice
 : @param $edition The edition in which an error occurs
 : @param $mode The current edition
 : @param $id The target id
 : @return The modal div
 :)
declare function single:make-typo-corr-modal($node as element(tei:choice),
$edition as xs:string, $mode as xs:string, $id as xs:string, $file as xs:string)
as element(xhtml:div)
{
  <div class="modal fade" role="dialog" id="{$id}">
    <div class="modal-dialog modal-typo">
      <div class="modal-content">
        <div class="modal-header">
          <button type="button" class="close" data-dismiss="modal">
            × <!-- times -->
          </button>
          {
            if ($node/tei:corr/@type = "authorial") then
              let $sic :=
                if ($node/tei:sic[not(child::node() or child::node())]) then
                  <em>&#248;</em>
                else
                  single:transform($node/tei:sic, $mode, $file)
              return
                (string("Autorschaftliche Korrektur von: "),
                $sic,
                string(" (" || $edition || ")"))
            else
              let $sic :=
                if ($node/tei:sic[not(child::* or child::node())]) then
                  <em>&#248;</em>
                else
                  single:transform($node/tei:sic, $mode, $file)
              return
                (string("Editorische Korrektur von: "),
                $sic,
                string(" (" || $edition || ")"))
          }
        </div>
      </div>
    </div>
  </div>
};


declare function single:is-next-text-in-app($node as node()*) as xs:boolean
{
  if ($node/following::text()[1][normalize-space(.) != ""][ancestor::tei:app])then
    true()
  else
    false()
};

declare function single:is-next-app-omitted($node as node()*, $mode as xs:string)
as xs:boolean
{
  let $next-app := $node/following-sibling::*[1][self::tei:app]
  return
    if ($next-app/child::tei:rdg[matches(@wit, $mode) and @type = "om"]) then
      true()
    else
      false()
};

(:~
 : In cases like tei:join several references are stated in a @target. This
 : function returns all corresponding nodes.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $target The @target of a e.g. tei:join
 : @param $mode The current edition
 : @return The nodes the @target refers to
 :)
declare function single:apply-join($node as element(tei:join), $mode as xs:string,
$file as xs:string) as node()*
{
    let $apply-only-children :=
        if ($node/@scope = "root") then
            false()
        else
            true()
    return
        for $copyOf in tokenize($node/@target, " ")
            let $applied-node := single:apply-corresp-node($node, $copyOf, $mode, $file, $apply-only-children)
            let $first-text := $applied-node[1]/descendant-or-self::text()[1]
            let $conjunction :=
              if (starts-with($first-text, ",")) then
                text{""}
              else
                text{" "}
            return ($conjunction, $applied-node)
};

(:~
 : Creates a random ID of 40 letters consisting of Basic Latin letters and
 : numbers. This function is needed since the usage of generate-id() (which we
 : originally used for the linking between icons and a modal dialogue) lead to
 : ID duplettes.
 :)
declare function single:make-random-id() as xs:string
{
    let $charset := "ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijklmnopqrstuvwxyz0123456789"
    let $id :=
        for $iii in 1 to 40 return
            let $number := util:random(40)
            return
                substring($charset, $number, 1)
    return string-join($id, "")
};
